#!/bin/sh
# IBM WebSphere Application Server Patterns
#
# Copyright IBM Corp. 2015
# All rights reserved.
# US Government Users Restricted Rights -
# Use, duplication or disclosure restricted
# by GSA ADP Schedule Contract with IBM Corp.
#
# Licensed Materials-Property of IBM
#
# A Script Package for tuning WAS profiles
#
# usage: sh WebSphereTuning.sh  <parameters>
#

# run from the directory this file is located in
SCRIPTDIR=`pwd`
echo "Current directory $SCRIPTDIR"

. /etc/virtualimage.properties

. ${SCRIPTDIR}/webspheretuning.properties

# IHS Check
if [ -z "$PROFILES_HOME" ]; then
  	echo "PROFILES_HOME is not defined in virtualimage.properties, WebSphere Tuning script does not a support this Server Type"
  	exit 181
fi

# There is no nodeagent on custom node
if [ $PROFILE_TYPE == 'custom' ] ; then
	if [ ! -d "${PROFILE_ROOT}/config/cells/${CELL_NAME}/nodes/${NODE_NAME}/servers/nodeagent" ]; then
		echo "There is no nodeagent on the virtual machine.  WebSphere Tuning script does not support this server type."
		exit 182
	fi
fi

WAS_PROFILES=$PROFILES_HOME
WHV_USER=`cat $WAS_PROFILES/.ibm/username`
if [ -z "$WHV_USER" ]; then
    WHV_USER=virtuser
fi

echo "Tuning the WebSphere profile configuration"

#Constants used during the JVM Tuning 
# Vars in property file.
#MIN_RAM=256
#MAX_RAM=4096
#OS_RAM=1280
#THREADS_PER_CPU=10

profile_num=1

# Check if it is a Linux or Power AIX 

if [ -f /etc/SuSE-release ] || [ -f /etc/redhat-release ] ; then
	echo "It is a Linux system"
	NUM_CPU=`cat /proc/cpuinfo | grep processor | wc -l`
	AMT_MEM=`cat /proc/meminfo | grep MemTotal | awk ' {print $2;}'` #in KBs
    #get system information for tuning in 390
	#NUM_CPU=`cat /proc/cpuinfo | grep ^processor | wc -l`
	#AMT_MEM=`cat /proc/meminfo | grep MemTotal | awk ' {print $2;}'` #in KBs
elif [ -f /etc/motd ]; then
    echo "It is a Power AIX system"
	NUM_CPU=`lsdev -Ccprocessor | grep -ci proc`
	AMT_MEM=`bootinfo -r`
fi

#Echo some variables/constants
echo "Minimum RAM = $MIN_RAM"
echo "Maximum RAM = $MAX_RAM"
echo "Memory overhead = $OS_RAM"
echo "Threads per CPU = $THREADS_PER_CPU"
echo "NUM of CPUs = $NUM_CPU"
echo "AMT of Memory = $AMT_MEM"
if [ $type ] ; then
    echo "type=$type"
fi

type=$PROFILE_TYPE
echo "Profile type = $type"
#-----------------------------------------------
# tune the servers registered  to the adminagent
#-----------------------------------------------
if [ $type == 'adminagent' ] ; then

    echo "Tuning administrative agent registered server profiles"

    profile_num=$PROFILE_NUMBER
    echo "profile_num = $profile_num"
    
    THREADS=$(( $NUM_CPU*$THREADS_PER_CPU ))
   
    if [ $profile_num -eq 1 ] ; then
        HEAP_SIZE=$( echo $AMT_MEM $OS_RAM | awk ' { printf("%.0f", $1 / 1024 - $2) }' ) #in MBs
    else
        HEAP_SIZE=$( echo $AMT_MEM $OS_RAM $profile_num | awk ' { printf("%.0f", (($1 / 1024) - $2) / $3) }' ) #in MBs
        THREADS=$(echo $THREADS $profile_num | awk ' { printf("%.0f", $1 / $2) }' ) #divide threads between JVMs
    fi
    if (( $HEAP_SIZE < $MIN_RAM )) ; then
        echo "Heap size does not meet the minimum size requirement: $HEAP_SIZE"
        HEAP_SIZE=$MIN_RAM
        exit 180     
    fi
    if (( $HEAP_SIZE > $MAX_RAM )) ; then
        HEAP_SIZE=$MAX_RAM
    fi
  
    # tune any app servers defined
    profile_index=1
    while [ $profile_index -le $profile_num ]
    do
        echo "Tune $PROFILE_NAME_PREFIX"0"$profile_index"
        PROF_NAME=$PROFILE_NAME_PREFIX"0"$profile_index
        SRV_NODE_NAME=$PROFILE_NAME_PREFIX"0"$profile_index"Node"
        SERVER_NAME=server1

        CMD="$WAS_PROFILES/$PROF_NAME/bin/wsadmin.sh -conntype NONE -lang jython -f $SCRIPTDIR/tuneWASProfile.py $SRV_NODE_NAME $SERVER_NAME $THREADS $HEAP_SIZE"
        echo "Running command: ${CMD}"
        su $WHV_USER -c "$CMD"
        
        let profile_index=$profile_index+1
    done
    
    echo "The adminagent registered server profiles have been tuned"
fi
#---------------------
# tune the odr server
#---------------------
if [ $type == 'odr' ] ; then

    echo "Tuning the on demand router"
    PROF_NAME=$PROFILE_NAME
    SERVER_NAME=odr

    HEAP_SIZE=$( echo $AMT_MEM $OS_RAM | awk ' { printf("%.0f", $1 / 1024 - $2) }' ) #in MBs
    if (( $HEAP_SIZE < $MIN_RAM )) ; then
        echo "Heap size does not meet the minimum size requirement: $HEAP_SIZE"
        HEAP_SIZE=$MIN_RAM
        exit 180
    fi
    if (( $HEAP_SIZE > $MAX_RAM )) ; then
        HEAP_SIZE=$MAX_RAM
    fi

    CMD="$WAS_PROFILES/$PROF_NAME/bin/wsadmin.sh -lang jython -f $SCRIPTDIR/tuneODRProfile.py $NODE_NAME $SERVER_NAME $HEAP_SIZE"
    echo "Running command: ${CMD}"
    su $WHV_USER -c "$CMD"

    echo "The ODR has been tuned"
fi
#-------------------------------------
# tune the standalone and dmgr servers
#-------------------------------------
if [ $type == 'standalone' -o $type == 'dmgr' ] ; then

    if [ $type == 'standalone' ] ; then
        echo "Tuning the default application server"
        #PROF_NAME=DefaultAppSrv01
        PROF_NAME=$PROFILE_NAME
        SERVER_NAME=server1
    else
        echo "Tuning the default deployment manager"
        #PROF_NAME=DefaultDmgr01
        PROF_NAME=$PROFILE_NAME
        SERVER_NAME=dmgr
    fi

    THREADS=$(( $NUM_CPU*$THREADS_PER_CPU ))
    HEAP_SIZE=$( echo $AMT_MEM $OS_RAM | awk ' { printf("%.0f", $1 / 1024 - $2) }' ) #in MBs
    if (( $HEAP_SIZE < $MIN_RAM )) ; then
        echo "Heap size does not meet the minimum size requirement: $HEAP_SIZE"
        HEAP_SIZE=$MIN_RAM
        exit 180
    fi
    if (( $HEAP_SIZE > $MAX_RAM )) ; then
        HEAP_SIZE=$MAX_RAM
    fi

    CMD="$WAS_PROFILES/$PROF_NAME/bin/wsadmin.sh -conntype NONE -lang jython -f $SCRIPTDIR/tuneWASProfile.py $NODE_NAME $SERVER_NAME $THREADS $HEAP_SIZE"
    echo "Running command: ${CMD}"
    su $WHV_USER -c "$CMD"
    
fi
#------------------------------------------------
# tune the nodeagent and all servers in that node 
#------------------------------------------------
if [ $type == 'custom' ] ; then

    echo "Tuning the custom node and servers"
    #PROF_NAME=DefaultCustom01
    PROF_NAME=$PROFILE_NAME
    SERVER_NAME=nodeagent

    CMD="$WAS_PROFILES/$PROF_NAME/bin/wsadmin.sh  -lang jython -f $SCRIPTDIR/tuneOtherJVM.py $NODE_NAME $AMT_MEM $NUM_CPU $MIN_RAM $MAX_RAM $OS_RAM $THREADS_PER_CPU"
    echo "Running command: ${CMD}"
    su $WHV_USER -c "$CMD"

fi

if [ $type == "liberty" ] || [ $type == "libertycontroller" ] || [ $type == "libertyhost" ]; then
   echo "Liberty Profile does not have any tuning that is done"
fi

echo "Done tuning the WebSphere profile configuration"

